/* libutf8/src/lib/ctype.h
 *
 *  (c)2006, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

/*! \defgroup ctype Character classification

This module contains functions for character classification. These are basically an extension of the
\c is* functions defined in \c &lt;ctype.h&gt;.

\todo There are many char classification functions that haven't been implemented yet. These won't be
    implemented until they can be done in a proper, Unicode-safe fashion.

 */
/*!@{*/



/// Returns \c true if \a ch can be represented in ASCII.
bool utf8_isascii(wchar_t ch);

/// Returns \c true if \a ch is whitespace.
bool utf8_isspace(wchar_t ch);

/*! \brief Returns \c true if \a ch is a valid UCS-4 character.

\param ch The character to classify.
\retval true If \a ch is a valid UCS-4 character.
\retval false If \a ch is not a valid UCS-4 character.

This function will examine a \c wchar_t value and determine whether or not it is a valid UCS-4
character. Valid characters lie in the range 0&ndash;0x7FFFFFFF but exclude:
\li the UTF-16 surrogate code points (U+D800&ndash;U+DFFF, inclusive)
\li the invalid code points U+FFFE and U+FFFF

*/
bool utf8_isvalid(wchar_t ch);



/*!@}*/

/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
*/
